/* Emacs style mode select   -*- C++ -*- 
 *-----------------------------------------------------------------------------
 *
 * $Id: l_joy.c,v 1.12 1999/10/31 09:12:17 cphipps Exp $
 *
 *  PowerDoom, a multi-platform Doom port to the PowerPak GSDK
 *  based on lsdldoom, the SDL Port of the BOOM port of DOOM
 *  Copyright (C) 1999 by Colin Phipps
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 *  02111-1307, USA.
 *
 * DESCRIPTION:
 *   Joystick handling for Linux
 *
 *-----------------------------------------------------------------------------
 */

#ifndef lint
static const char rcsid[] = "$Id: l_joy.c,v 1.12 1999/10/31 09:12:17 cphipps Exp $";
#endif /* lint */

#include "doomdef.h"
#include "doomtype.h"
#include "m_argv.h"
#include "d_event.h"
#include "d_main.h"
#include "i_joy.h"
#include "lprintf.h"

#undef NO_JOY_CODE

#ifndef HAVE_LINUX_JOYSTICK_H
#define NO_JOY_CODE
#else

#include <PowerPak/powerinput.h>

#endif

int joyopen;
int joyleft;
int joyright;
int joyup;
int joydown;

int usejoystick;

void I_EndJoystick(void)
{
#ifndef NO_JOY_CODE
  lprintf(LO_DEBUG, "I_EndJoystick : closing joystick\n");
  PI_CloseJoystick(usejoystick-1);
#endif
}

#ifndef NO_JOY_CODE
static int I_WaitButton(void)
{
  do {
    PP_Wait(10);
  }while (!PI_AreAnyJoyButtonsDown(usejoystick-1));
  while (PI_AreAnyJoyButtonsDown(usejoystick-1)) {
    PP_Wait(10);
  }
  return 1;
}
#endif


void I_PollJoystick(void)
{
  int button;
  event_t ev;

#ifndef NO_JOY_CODE
  if (!usejoystick || (joyopen == -1)) return;

#ifndef DOSDOOM
    button &= 0x00;
    button |= (PI_IsJoyButtonDown(usejoystick-1, 0) << 0);
    button |= (PI_IsJoyButtonDown(usejoystick-1, 1) << 1);
    button |= (PI_IsJoyButtonDown(usejoystick-1, 2) << 2);
    button |= (PI_IsJoyButtonDown(usejoystick-1, 3) << 3);

    ev.type = ev_joystick;
    ev.data1 = button;
#else
    ev.type = ev_analogue;
    ev.data1 = ev.data3 = 0;
#endif

    ev.data2=(PI_JoyX(usejoystick-1) < joyleft) ? -1 : ((PI_JoyX(usejoystick-1) > joyright) ? 1 : 0);

#ifndef DOSDOOM
    ev.data3 = 
#else
    ev.data4 = 
#endif
      (PI_JoyY(usejoystick-1) < joyup) ? -1 : ((PI_JoyY(usejoystick-1) > joydown ) ? 1 : 0);
    D_PostEvent(&ev);

#ifdef DOSDOOM
    /* Buttons handled as keypress events */
    {
      int button_num;

      for (button_num = 0; button_num<3; button_num++) {
	  ev.type = (PI_IsJoyButtonDown(usejoystick-1, button_num) ? ev_keydown : ev_keyup;
	  ev.data1 = KEYD_JOY1 + button_num;
	}
      }
    }
#endif

#endif
}

void I_InitJoystick(void)
{
   int joyt, joyl, joyb, joyr, joycx, joycy;

#ifndef NO_JOY_CODE
  const char* fname = "I_InitJoystick : ";

  if (!usejoystick) return;
  if (M_CheckParm("-nojoy") || (usejoystick>9) || (usejoystick<0)) {
    if ((usejoystick > 9) || (usejoystick < 0))
      lprintf(LO_WARN, "%sinvalid joystick %d\n", fname, usejoystick);
    else
      lprintf(LO_INFO, "%suser disabled\n", fname);
    joyopen = -1; return;
  }

  joyopen = PI_OpenJoystick(usejoystick-1);
  if (joyopen == -1)
    lprintf(LO_DEBUG, "I_InitJoystick : Couldn't open joystick %d\n", usejoystick);
  else {
    atexit(I_EndJoystick);
    lprintf(LO_INFO, "%sopened joystick %d\n", fname, usejoystick);
    if ((joyup == joydown) || (joyleft == joyright)) {
      printf("Invalid joystick calibration. Calbrating now...\n");

      printf("Move joystick to top-left and press a button\n");
      fflush(stdin);
      if (!I_WaitButton()) return;
      joyt = PI_JoyY(usejoystick-1); joyl = PI_JoyX(usejoystick-1);

      printf("Move joystick to bottom right and press a button\n");
      fflush(stdin);
      if (!I_WaitButton()) return;
      joyb = PI_JoyY(usejoystick-1); joyr = PI_JoyX(usejoystick-1);

      printf("Move joystick to centre and press a button\n");
      fflush(stdin);
      if (!I_WaitButton()) return;
      joycy = PI_JoyY(usejoystick-1); joycx = PI_JoyX(usejoystick-1);

      joyleft = (joyl + joycx)/2; joyright = (joyr + joycx)/2;
      joyup   = (joyt + joycy)/2; joydown  = (joyb + joycy)/2;
    }
  }
#endif
}

/*
 * $Log: l_joy.c,v $
 * Revision 1.12  1999/10/31 09:12:17  cphipps
 * Fix comment typo
 *
 * Revision 1.11  1999/10/31 09:10:10  cphipps
 * Applied path from Benjamin McGee to reopen the joystick device on read
 * errors.
 * Same patch fixes a typo in the joystick calibration stuff.
 * Changed a lot of fprintf's to lprintf's.
 * Fixed last C++ comment.
 *
 * Revision 1.10  1999/10/12 13:01:11  cphipps
 * Changed header to GPL
 *
 * Revision 1.9  1999/09/10 18:08:11  cphipps
 * Modified to use autoconf generated define to decide whether joystick header is
 * available. Thanks to Josh Parsons.
 *
 * Revision 1.8  1999/01/04 19:30:00  cphipps
 * Remove duplicate joystick variable instances
 *
 * Revision 1.7  1998/12/16 22:27:12  cphipps
 * Fix warnings when NO_JOY_CODE is defined by making the static unused functions not even be defined in that case
 *
 */





