/* Emacs style mode select   -*- C++ -*- 
 *-----------------------------------------------------------------------------
 *
 * $Id: l_video_x.c,v 1.27 1999/10/12 13:01:11 cphipps Exp $
 *
 *  PowerDoom, a multi-platform Doom port to the PowerPak GSDK
 *  based on lsdldoom, the SDL Port of the BOOM port of DOOM
 *  Copyright (C) 1993-1996 by id Software
 *  Copyright (C) 1999 by Colin Phipps
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 *  02111-1307, USA.
 *
 * DESCRIPTION:
 *	DOOM graphics stuff for PowerPak
 *
 *-----------------------------------------------------------------------------
 */

static const char
rcsid[] = "$Id: l_video_x.c,v 1.27 1999/10/12 13:01:11 cphipps Exp $";

#include <stdlib.h>
#include <unistd.h>

#include <PowerPak/powerdraw.h>
#include <PowerPak/powerinput.h>

#include "i_system.h"
#include "m_argv.h"
#include "doomstat.h"
#include "doomdef.h"
#include "doomtype.h"
#include "v_video.h"
#include "r_draw.h"
#include "d_main.h"
#include "d_event.h"
#include "l_video_trans.h"
#include "i_joy.h"
#include "i_video.h"
#include "z_zone.h"
#include "s_sound.h"
#include "sounds.h"
#include "w_wad.h"
#include "lprintf.h"

void (*R_DrawColumn)(void);
void (*R_DrawTLColumn)(void);

extern void M_QuitDOOM(int choice);

int use_vsync = 0; // Included not to break m_misc, but not relevant to PowerPak

// This is the pointer to the buffer to blit
pval     *      out_buffer = NULL;

// Common const strings
static const char doomcaption[] = { "PowerDoom" };

////////////////////////////////////////////////////////////////////////////
// Input code 
int             leds_always_off = 0; // Expected by m_misc, not relevant 

// Mouse handling
extern int     usemouse;        // config file var
static boolean grabMouse;       // internal var

/////////////////////////////////////////////////////////////////////////////////
// Keyboard handling

//
//  Translates the key currently in key
//

static int I_TranslateKey(PI_KeySymbol* key)
{
  int rc = 0;

  switch (key->sym) {
  case PIK_LEFT:	rc = KEYD_LEFTARROW;	break;
  case PIK_RIGHT:	rc = KEYD_RIGHTARROW;	break;
  case PIK_DOWN:	rc = KEYD_DOWNARROW;	break;
  case PIK_UP:		rc = KEYD_UPARROW;	break;
  case PIK_ESCAPE:	rc = KEYD_ESCAPE;	break;
  case PIK_RETURN:	rc = KEYD_ENTER;	break;
  case PIK_TAB:	rc = KEYD_TAB;		break;
  case PIK_F1:		rc = KEYD_F1;		break;
  case PIK_F2:		rc = KEYD_F2;		break;
  case PIK_F3:		rc = KEYD_F3;		break;
  case PIK_F4:		rc = KEYD_F4;		break;
  case PIK_F5:		rc = KEYD_F5;		break;
  case PIK_F6:		rc = KEYD_F6;		break;
  case PIK_F7:		rc = KEYD_F7;		break;
  case PIK_F8:		rc = KEYD_F8;		break;
  case PIK_F9:		rc = KEYD_F9;		break;
  case PIK_F10:	rc = KEYD_F10;		break;
  case PIK_F11:	rc = KEYD_F11;		break;
  case PIK_F12:	rc = KEYD_F12;		break;
  case PIK_BACKSPACE:
  case PIK_DELETE:	rc = KEYD_BACKSPACE;	break;
  case PIK_PAUSE:	rc = KEYD_PAUSE;	break;
  case PIK_EQUALS:	rc = KEYD_EQUALS;	break;
  case PIK_MINUS:	rc = KEYD_MINUS;	break;
  case PIK_KP0:	rc = KEYD_KEYPAD0;	break;
  case PIK_KP1:	rc = KEYD_KEYPAD1;	break;
  case PIK_KP2:	rc = KEYD_KEYPAD2;	break;
  case PIK_KP3:	rc = KEYD_KEYPAD3;	break;
  case PIK_KP4:	rc = KEYD_KEYPAD4;	break;
  case PIK_KP5:	rc = KEYD_KEYPAD0;	break;
  case PIK_KP6:	rc = KEYD_KEYPAD6;	break;
  case PIK_KP7:	rc = KEYD_KEYPAD7;	break;
  case PIK_KP8:	rc = KEYD_KEYPAD8;	break;
  case PIK_KP9:	rc = KEYD_KEYPAD9;	break;
  case PIK_KP_PLUS:	rc = KEYD_KEYPADPLUS;	break;
  case PIK_KP_MINUS:	rc = KEYD_KEYPADMINUS;	break;
  case PIK_KP_DIVIDE:	rc = KEYD_KEYPADDIVIDE;	break;
  case PIK_KP_MULTIPLY: rc = KEYD_KEYPADMULTIPLY; break;
  case PIK_KP_ENTER:	rc = KEYD_KEYPADENTER;	break;
  case PIK_LSHIFT:
  case PIK_RSHIFT:	rc = KEYD_RSHIFT;	break;
  case PIK_LCTRL:
  case PIK_RCTRL:	rc = KEYD_RCTRL;	break;
  case PIK_LALT:
  case PIK_LMETA:
  case PIK_RALT:
  case PIK_RMETA:	rc = KEYD_RALT;		break;
  default:		rc = key->sym;		break;
  }

  return rc;

}

// Null keyboard translation to satisfy m_misc.c
int I_DoomCode2ScanCode(int c)
{
  return c;
}

int I_ScanCode2DoomCode(int c)
{
  return c;
}

/////////////////////////////////////////////////////////////////////////////////
// Main input code

static void I_GetEvent()
{
  event_t event;

  switch (Event.type) {
  case PI_KEYDOWN:
    event.type = ev_keydown;
    event.data1 = I_TranslateKey(&Event.key.keysym);
    D_PostEvent(&event);
    break;

  case PI_KEYUP:
  {
    event.type = ev_keyup;
    event.data1 = I_TranslateKey(&Event.key.keysym);
    D_PostEvent(&event);
  }
  break;

  case PI_MOUSEBUTTONDOWN:
  case PI_MOUSEBUTTONUP:

  if (usemouse)
  { Uint8 buttonstate;
    buttonstate = PI_GetMouseState(NULL, NULL);
    event.type = ev_mouse;
    event.data1 = 0
      | (PI_WasMouseButtonDown(buttonstate, 1) ? 1 : 0)
      | (PI_WasMouseButtonDown(buttonstate, 2) ? 2 : 0)
      | (PI_WasMouseButtonDown(buttonstate, 3) ? 4 : 0);
    event.data2 = event.data3 = 0;
    D_PostEvent(&event);
  }
  break;

  case PI_MOUSEMOTION:
  /* Ignore mouse warp events */
  if (usemouse &&
      ((Event.motion.x != Screen->w/2)||(Event.motion.y != Screen->h/2)))
  {
    /* Warp the mouse back to the center */
    if (grabMouse) {
      if ( (Event.motion.x < ((Screen->w/2)-(Screen->w/4))) ||
           (Event.motion.x > ((Screen->w/2)+(Screen->w/4))) ||
           (Event.motion.y < ((Screen->h/2)-(Screen->h/4))) ||
           (Event.motion.y > ((Screen->h/2)+(Screen->h/4))) )
        PI_MoveMouseCursor(Screen->w/2, Screen->h/2);
    }
    event.type = ev_mouse;
    event.data1 = 0
      | (PI_WasMouseButtonDown(Event.motion.state, 1) ? 1 : 0)
      | (PI_WasMouseButtonDown(Event.motion.state, 2) ? 2 : 0)
      | (PI_WasMouseButtonDown(Event.motion.state, 3) ? 4 : 0);
    event.data2 = Event.motion.xrel << 2;
    event.data3 = -Event.motion.yrel << 2;
    D_PostEvent(&event);
  }
  break;

  case PI_QUIT:
    S_StartSound(NULL, sfx_swtchn);
    M_QuitDOOM(0);

  default:
    break;
  }
}

//
// I_StartTic
//
void I_StartTic (void)
{
  while ( PI_PollEvent() )
    I_GetEvent();

  I_PollJoystick();
}

//
// I_StartFrame
//
void I_StartFrame (void)
{
}

//
// I_InitInputs
//

static void I_InitInputs(void)
{
  // check if the user wants to grab the mouse (quite unnice)
  grabMouse = M_CheckParm("-nomouse") ? false : 
    usemouse ? true : false;

  I_InitJoystick();
}
/////////////////////////////////////////////////////////////////////////////

// I_SkipFrame
//
// Returns true if it thinks we can afford to skip this frame

static inline boolean I_SkipFrame(void)
{
  static int frameno;

  frameno++;
  switch (gamestate) {
  case GS_LEVEL:
    if (!paused)
      return false;
  default:
    // Skip odd frames
    return (frameno & 1) ? true : false;
  }
}

///////////////////////////////////////////////////////////
// Palette stuff.
//
static void I_UploadNewPalette(int pal)
{
  // This is used to replace the current 256 colour cmap with a new one
  // Used by 256 colour PseudoColor modes

  // Array of PD_Color structs used for setting the 256-colour palette
  static PD_Palette colours;
  static int cachedgamma;
  static size_t num_pals;

  if ((colours == NULL) || (cachedgamma != usegamma)) {
    int            lump = W_GetNumForName("PLAYPAL");
    const byte *palette = W_CacheLumpNum(lump);
    register const byte *const gtable = gammatable[cachedgamma = usegamma];
    register int i;

    num_pals = W_LumpLength(lump) / (3*256);
    num_pals *= 256;

    if (!colours) {
      // First call - allocate and prepare colour array
      colours = malloc(sizeof(*colours)*num_pals);
    }

    // set the colormap entries
    for (i=0 ; i<num_pals ; i++) {
      colours[i].r = gtable[palette[0]];
      colours[i].g = gtable[palette[1]];
      colours[i].b = gtable[palette[2]];
      palette += 3;
    }
  
    W_UnlockLumpNum(lump);
    num_pals/=256;
  }

#ifdef RANGECHECK
  if (pal >= num_pals) 
    I_Error("I_UploadNewPalette: Palette number out of range (%d>=%d)", 
	    pal, num_pals);
#endif
  
  // store the colors to the current display
  PD_SetPalette(Screen, colours+256*pal);
}

//////////////////////////////////////////////////////////////////////////////
// Graphics API

void I_ShutdownGraphics(void)
{
  fprintf(stderr, "I_ShutdownGraphics : ");

  // Free internal structures
  if (pixelvals != NULL) free(pixelvals);
  I_EndImageTranslation();
  fprintf(stderr, "\n");
}

//
// I_UpdateNoBlit
//
void I_UpdateNoBlit (void)
{
}

//
// I_FinishUpdate
//
void I_FinishUpdate (void)
{
  if (I_SkipFrame()) return;

#ifdef MONITOR_VISIBILITY
  if (!PP_IsAppActive()) {
    return;
  }
#endif
  
  // scales the screen size before blitting it
  if (expand_buffer)
    (*I_ExpandImage)(out_buffer, screens[0]);
  
  // Update the display buffer (flipping video pages if supported)
  PD_ShowDoubleBuffer();
}

//
// I_ReadScreen
//
void I_ReadScreen (byte* scr)
{
  memcpy(scr, screens[0], SCREENWIDTH*SCREENHEIGHT);
}

//
// I_SetPalette
//
void I_SetPalette (int pal)
{
  if (true_color) {
    int            lump = W_GetNumForName("PLAYPAL");
    const byte *palette = W_CacheLumpNum(lump);
    I_SetPaletteTranslation(palette + (3*256)*pal);
    W_UnlockLumpNum(lump);
  } else
    I_UploadNewPalette(pal);
}

// I_PreInitGraphics

void I_PreInitGraphics(void)
{
}

// CPhipps -
// I_SetRes
// Sets the screen resolution, possibly using the supplied guide

void I_SetRes(unsigned int width, unsigned int height)
{
#ifdef HIGHRES
  SCREENWIDTH = 
#endif
    (width+3) & ~3;

#ifdef HIGHRES
  SCREENHEIGHT = 
#endif
    (height+3) & ~3;

  if (SCREENWIDTH == 320) {
    R_DrawColumn = R_DrawColumn_Normal;
    R_DrawTLColumn = R_DrawTLColumn_Normal;
  } else {
    R_DrawColumn = R_DrawColumn_HighRes;
    R_DrawTLColumn = R_DrawTLColumn_HighRes;
  }
  printf("I_SetRes: Using resolution %dx%d\n", SCREENWIDTH, SCREENHEIGHT);
}

void I_InitGraphics(void)
{
  int           w, h;
  int		n;

  {  
    static int		firsttime=1;
    
    if (!firsttime) {
      return;
    }
    firsttime = 0;
  }

  { // Check for screen enlargement
    char str[3] = { '-', 0, 0 };

    for (n=1; n<4; n++) {
      str[1] = n + '0';
      if (M_CheckParm(str)) multiply = n;
    }
  }
  
  w = SCREENWIDTH * multiply;
  h = SCREENHEIGHT * multiply;

  /* Initialize the library */
  PP_Init();
  PD_Init();

  /* Initialize 640x480x16bpp graphics mode */
  if ( M_CheckParm("-fullscreen") ) {
     PD_SetGFXMode(w, h, 0, PD_FULLSCREENDIRECT);
  }else{
     PD_SetGFXMode(w, h, 0, PD_DEFAULTDIRECT);
  }

  dest_bpp = Screen->format->BitsPerPixel;

  PD_SetCaption(doomcaption);

  I_InitImageTranslation();
  if (true_color) {
    // Set up colour shifts
    I_SetColourShift(Screen->format->Rmask, &redshift);
    I_SetColourShift(Screen->format->Gmask, &greenshift);
    I_SetColourShift(Screen->format->Bmask, &blueshift);
  }

  lprintf(LO_INFO,"I_InitGraphics:");
  lprintf(LO_INFO, " PowerPak %d bpp %s, scale x%d\n", dest_bpp,
	  Screen->format->palette ? "PseudoColor" : "TrueColor", multiply);

  // Get the info needed to render to the display
  out_buffer = (pval *)Screen->pixels;
  if (!expand_buffer) {
    // Render directly into display memory
    Z_Free(screens[0]);
    screens[0] = (unsigned char *) (Screen->pixels);
  }

  atexit(I_ShutdownGraphics);

  // Initialize the input system
  I_InitInputs();

  // Hide pointer while over this window
  PI_HideMouseCursor();
}

//
// $Log: l_video_x.c,v $
// Revision 1.1  1998/10/27 07:50:11  cphipps
// Initial revision
//
